#pragma once
#include <string>
#include <map>
#include <stdio.h>  //sprintf
#include <stdlib.h> //atoi

using namespace std;

class String : public string
{
public:
  String() {}
  String(const string & a) : string(a) {}
  String(const char* a)    : string(a) {}
  String(const int a)      { char s[20]; sprintf(s, "%d", a);  *this = s; }
  
  const char* asString (const char* defaults = "")   const { return this->empty() ? defaults : this->c_str(); }
  int         asInteger(const int defaults = 0)      const { return this->empty() ? defaults : atoi( this->c_str() ); }
  double      asDouble (const double defaults = 0.0) const { char **endptr;  return this->empty() ? defaults : strtod(this->c_str(), endptr); }
  bool        asBoolean(const bool defaults = false) const { return this->empty() ? defaults : (*this=="true" || *this=="yes" || *this=="on" || *this=="1"); }
  
  String& trim() {
    const char* t = (char*)" \t\n\r\f";
    this->erase(0, this->find_first_not_of(t));
    this->erase(this->find_last_not_of(t)+1);
    return *this;
  }
  
  void sscanf(const char *format, String * a, String * b = NULL) {
    char c1[256] = {0};
    char c2[256] = {0};
    ::sscanf(this->c_str(), format, c1, c2);
    *a = c1;
    if(b) *b = c2;
  }  
};

class Section : public map<String, String>
{
public:
  const char* join(const char* = " ");
};

class Sections : public map<String, Section>
{
};

class Conf : public Sections
{
public:
  Conf(const char* name = 0, const char* rem = "#;");
  virtual ~Conf();                                                             
  
  const char* readString (const char* section, const char* key, const char*  defaults = "" );
  int         readInteger(const char* section, const char* key, const int    defaults = 0  );
  double      readDouble (const char* section, const char* key, const double defaults = 0.0);
  bool        readBoolean(const char* section, const char* key, const bool   defaults = false);
  Section&    readSection(const char* section);
  
  bool        exist      () const { return d_exist; }
  const char* name       () const { return d_name; } 
  unsigned int size      () const { return d_size; }
private:
  static bool module_name(char *);
  bool        open       (const char* = 0);
  bool        guess      (const char* = 0);
  
  bool d_exist;
  unsigned int d_size;
  char *d_name;
  char *d_rem;
};
